from .base import BaseClient as __BaseClient
from typing import List as _List


class GetPricingResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: PriceList = PriceList(data["payload"])
        else:
            self.payload: PriceList = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetOffersResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetOffersResult = GetOffersResult(data["payload"])
        else:
            self.payload: GetOffersResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetOffersResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceID" in data:
            self.MarketplaceID: str = str(data["MarketplaceID"])
        else:
            self.MarketplaceID: str = None
        if "ASIN" in data:
            self.ASIN: str = str(data["ASIN"])
        else:
            self.ASIN: str = None
        if "SKU" in data:
            self.SKU: str = str(data["SKU"])
        else:
            self.SKU: str = None
        if "ItemCondition" in data:
            self.ItemCondition: ConditionType = ConditionType(data["ItemCondition"])
        else:
            self.ItemCondition: ConditionType = None
        if "status" in data:
            self.status: str = str(data["status"])
        else:
            self.status: str = None
        if "Identifier" in data:
            self.Identifier: ItemIdentifier = ItemIdentifier(data["Identifier"])
        else:
            self.Identifier: ItemIdentifier = None
        if "Summary" in data:
            self.Summary: Summary = Summary(data["Summary"])
        else:
            self.Summary: Summary = None
        if "Offers" in data:
            self.Offers: OfferDetailList = OfferDetailList(data["Offers"])
        else:
            self.Offers: OfferDetailList = None


class Price:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "status" in data:
            self.status: str = str(data["status"])
        else:
            self.status: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = str(data["SellerSKU"])
        else:
            self.SellerSKU: str = None
        if "ASIN" in data:
            self.ASIN: str = str(data["ASIN"])
        else:
            self.ASIN: str = None
        if "Product" in data:
            self.Product: Product = Product(data["Product"])
        else:
            self.Product: Product = None


class Product:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "Identifiers" in data:
            self.Identifiers: IdentifierType = IdentifierType(data["Identifiers"])
        else:
            self.Identifiers: IdentifierType = None
        if "AttributeSets" in data:
            self.AttributeSets: AttributeSetList = AttributeSetList(data["AttributeSets"])
        else:
            self.AttributeSets: AttributeSetList = None
        if "Relationships" in data:
            self.Relationships: RelationshipList = RelationshipList(data["Relationships"])
        else:
            self.Relationships: RelationshipList = None
        if "CompetitivePricing" in data:
            self.CompetitivePricing: CompetitivePricingType = CompetitivePricingType(data["CompetitivePricing"])
        else:
            self.CompetitivePricing: CompetitivePricingType = None
        if "SalesRankings" in data:
            self.SalesRankings: SalesRankList = SalesRankList(data["SalesRankings"])
        else:
            self.SalesRankings: SalesRankList = None
        if "Offers" in data:
            self.Offers: OffersList = OffersList(data["Offers"])
        else:
            self.Offers: OffersList = None


class IdentifierType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceASIN" in data:
            self.MarketplaceASIN: ASINIdentifier = ASINIdentifier(data["MarketplaceASIN"])
        else:
            self.MarketplaceASIN: ASINIdentifier = None
        if "SKUIdentifier" in data:
            self.SKUIdentifier: SellerSKUIdentifier = SellerSKUIdentifier(data["SKUIdentifier"])
        else:
            self.SKUIdentifier: SellerSKUIdentifier = None


class ASINIdentifier:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceId" in data:
            self.MarketplaceId: str = str(data["MarketplaceId"])
        else:
            self.MarketplaceId: str = None
        if "ASIN" in data:
            self.ASIN: str = str(data["ASIN"])
        else:
            self.ASIN: str = None


class SellerSKUIdentifier:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceId" in data:
            self.MarketplaceId: str = str(data["MarketplaceId"])
        else:
            self.MarketplaceId: str = None
        if "SellerId" in data:
            self.SellerId: str = str(data["SellerId"])
        else:
            self.SellerId: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = str(data["SellerSKU"])
        else:
            self.SellerSKU: str = None


class CompetitivePricingType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "CompetitivePrices" in data:
            self.CompetitivePrices: CompetitivePriceList = CompetitivePriceList(data["CompetitivePrices"])
        else:
            self.CompetitivePrices: CompetitivePriceList = None
        if "NumberOfOfferListings" in data:
            self.NumberOfOfferListings: NumberOfOfferListingsList = NumberOfOfferListingsList(
                data["NumberOfOfferListings"]
            )
        else:
            self.NumberOfOfferListings: NumberOfOfferListingsList = None
        if "TradeInValue" in data:
            self.TradeInValue: MoneyType = MoneyType(data["TradeInValue"])
        else:
            self.TradeInValue: MoneyType = None


class CompetitivePriceType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "CompetitivePriceId" in data:
            self.CompetitivePriceId: str = str(data["CompetitivePriceId"])
        else:
            self.CompetitivePriceId: str = None
        if "Price" in data:
            self.Price: PriceType = PriceType(data["Price"])
        else:
            self.Price: PriceType = None
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None
        if "subcondition" in data:
            self.subcondition: str = str(data["subcondition"])
        else:
            self.subcondition: str = None
        if "belongsToRequester" in data:
            self.belongsToRequester: bool = bool(data["belongsToRequester"])
        else:
            self.belongsToRequester: bool = None


class OfferListingCountType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "Count" in data:
            self.Count: int = int(data["Count"])
        else:
            self.Count: int = None
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None


class MoneyType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "CurrencyCode" in data:
            self.CurrencyCode: str = str(data["CurrencyCode"])
        else:
            self.CurrencyCode: str = None
        if "Amount" in data:
            self.Amount: float = float(data["Amount"])
        else:
            self.Amount: float = None


class SalesRankType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "ProductCategoryId" in data:
            self.ProductCategoryId: str = str(data["ProductCategoryId"])
        else:
            self.ProductCategoryId: str = None
        if "Rank" in data:
            self.Rank: int = int(data["Rank"])
        else:
            self.Rank: int = None


class PriceType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = MoneyType(data["LandedPrice"])
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = MoneyType(data["ListingPrice"])
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = MoneyType(data["Shipping"])
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = Points(data["Points"])
        else:
            self.Points: Points = None


class OfferType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "BuyingPrice" in data:
            self.BuyingPrice: PriceType = PriceType(data["BuyingPrice"])
        else:
            self.BuyingPrice: PriceType = None
        if "RegularPrice" in data:
            self.RegularPrice: MoneyType = MoneyType(data["RegularPrice"])
        else:
            self.RegularPrice: MoneyType = None
        if "FulfillmentChannel" in data:
            self.FulfillmentChannel: str = str(data["FulfillmentChannel"])
        else:
            self.FulfillmentChannel: str = None
        if "ItemCondition" in data:
            self.ItemCondition: str = str(data["ItemCondition"])
        else:
            self.ItemCondition: str = None
        if "ItemSubCondition" in data:
            self.ItemSubCondition: str = str(data["ItemSubCondition"])
        else:
            self.ItemSubCondition: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = str(data["SellerSKU"])
        else:
            self.SellerSKU: str = None


class Points:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "PointsNumber" in data:
            self.PointsNumber: int = int(data["PointsNumber"])
        else:
            self.PointsNumber: int = None
        if "PointsMonetaryValue" in data:
            self.PointsMonetaryValue: MoneyType = MoneyType(data["PointsMonetaryValue"])
        else:
            self.PointsMonetaryValue: MoneyType = None


class ItemIdentifier:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceId" in data:
            self.MarketplaceId: str = str(data["MarketplaceId"])
        else:
            self.MarketplaceId: str = None
        if "ASIN" in data:
            self.ASIN: str = str(data["ASIN"])
        else:
            self.ASIN: str = None
        if "SellerSKU" in data:
            self.SellerSKU: str = str(data["SellerSKU"])
        else:
            self.SellerSKU: str = None
        if "ItemCondition" in data:
            self.ItemCondition: ConditionType = ConditionType(data["ItemCondition"])
        else:
            self.ItemCondition: ConditionType = None


class Summary:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "TotalOfferCount" in data:
            self.TotalOfferCount: int = int(data["TotalOfferCount"])
        else:
            self.TotalOfferCount: int = None
        if "NumberOfOffers" in data:
            self.NumberOfOffers: NumberOfOffers = NumberOfOffers(data["NumberOfOffers"])
        else:
            self.NumberOfOffers: NumberOfOffers = None
        if "LowestPrices" in data:
            self.LowestPrices: LowestPrices = LowestPrices(data["LowestPrices"])
        else:
            self.LowestPrices: LowestPrices = None
        if "BuyBoxPrices" in data:
            self.BuyBoxPrices: BuyBoxPrices = BuyBoxPrices(data["BuyBoxPrices"])
        else:
            self.BuyBoxPrices: BuyBoxPrices = None
        if "ListPrice" in data:
            self.ListPrice: MoneyType = MoneyType(data["ListPrice"])
        else:
            self.ListPrice: MoneyType = None
        if "SuggestedLowerPricePlusShipping" in data:
            self.SuggestedLowerPricePlusShipping: MoneyType = MoneyType(data["SuggestedLowerPricePlusShipping"])
        else:
            self.SuggestedLowerPricePlusShipping: MoneyType = None
        if "BuyBoxEligibleOffers" in data:
            self.BuyBoxEligibleOffers: BuyBoxEligibleOffers = BuyBoxEligibleOffers(data["BuyBoxEligibleOffers"])
        else:
            self.BuyBoxEligibleOffers: BuyBoxEligibleOffers = None
        if "OffersAvailableTime" in data:
            self.OffersAvailableTime: str = str(data["OffersAvailableTime"])
        else:
            self.OffersAvailableTime: str = None


class OfferCountType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None
        if "fulfillmentChannel" in data:
            self.fulfillmentChannel: FulfillmentChannelType = FulfillmentChannelType(data["fulfillmentChannel"])
        else:
            self.fulfillmentChannel: FulfillmentChannelType = None
        if "OfferCount" in data:
            self.OfferCount: int = int(data["OfferCount"])
        else:
            self.OfferCount: int = None


class LowestPriceType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None
        if "fulfillmentChannel" in data:
            self.fulfillmentChannel: str = str(data["fulfillmentChannel"])
        else:
            self.fulfillmentChannel: str = None
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = MoneyType(data["LandedPrice"])
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = MoneyType(data["ListingPrice"])
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = MoneyType(data["Shipping"])
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = Points(data["Points"])
        else:
            self.Points: Points = None


class BuyBoxPriceType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None
        if "LandedPrice" in data:
            self.LandedPrice: MoneyType = MoneyType(data["LandedPrice"])
        else:
            self.LandedPrice: MoneyType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = MoneyType(data["ListingPrice"])
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = MoneyType(data["Shipping"])
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = Points(data["Points"])
        else:
            self.Points: Points = None


class OfferDetail:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MyOffer" in data:
            self.MyOffer: bool = bool(data["MyOffer"])
        else:
            self.MyOffer: bool = None
        if "SubCondition" in data:
            self.SubCondition: str = str(data["SubCondition"])
        else:
            self.SubCondition: str = None
        if "SellerFeedbackRating" in data:
            self.SellerFeedbackRating: SellerFeedbackType = SellerFeedbackType(data["SellerFeedbackRating"])
        else:
            self.SellerFeedbackRating: SellerFeedbackType = None
        if "ShippingTime" in data:
            self.ShippingTime: DetailedShippingTimeType = DetailedShippingTimeType(data["ShippingTime"])
        else:
            self.ShippingTime: DetailedShippingTimeType = None
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = MoneyType(data["ListingPrice"])
        else:
            self.ListingPrice: MoneyType = None
        if "Points" in data:
            self.Points: Points = Points(data["Points"])
        else:
            self.Points: Points = None
        if "Shipping" in data:
            self.Shipping: MoneyType = MoneyType(data["Shipping"])
        else:
            self.Shipping: MoneyType = None
        if "ShipsFrom" in data:
            self.ShipsFrom: ShipsFromType = ShipsFromType(data["ShipsFrom"])
        else:
            self.ShipsFrom: ShipsFromType = None
        if "IsFulfilledByAmazon" in data:
            self.IsFulfilledByAmazon: bool = bool(data["IsFulfilledByAmazon"])
        else:
            self.IsFulfilledByAmazon: bool = None
        if "IsBuyBoxWinner" in data:
            self.IsBuyBoxWinner: bool = bool(data["IsBuyBoxWinner"])
        else:
            self.IsBuyBoxWinner: bool = None
        if "IsFeaturedMerchant" in data:
            self.IsFeaturedMerchant: bool = bool(data["IsFeaturedMerchant"])
        else:
            self.IsFeaturedMerchant: bool = None


class SellerFeedbackType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "SellerPositiveFeedbackRating" in data:
            self.SellerPositiveFeedbackRating: float = float(data["SellerPositiveFeedbackRating"])
        else:
            self.SellerPositiveFeedbackRating: float = None
        if "FeedbackCount" in data:
            self.FeedbackCount: int = int(data["FeedbackCount"])
        else:
            self.FeedbackCount: int = None


class DetailedShippingTimeType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "minimumHours" in data:
            self.minimumHours: int = int(data["minimumHours"])
        else:
            self.minimumHours: int = None
        if "maximumHours" in data:
            self.maximumHours: int = int(data["maximumHours"])
        else:
            self.maximumHours: int = None
        if "availableDate" in data:
            self.availableDate: float = float(data["availableDate"])
        else:
            self.availableDate: float = None
        if "availabilityType" in data:
            self.availabilityType: str = str(data["availabilityType"])
        else:
            self.availabilityType: str = None


class ShipsFromType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "State" in data:
            self.State: str = str(data["State"])
        else:
            self.State: str = None
        if "Country" in data:
            self.Country: str = str(data["Country"])
        else:
            self.Country: str = None


class Error:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "code" in data:
            self.code: str = str(data["code"])
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = str(data["message"])
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = str(data["details"])
        else:
            self.details: str = None


class PriceList(list, _List["Price"]):
    def __init__(self, data):
        super().__init__([Price(datum) for datum in data])
        self.data = data


class AttributeSetList(list, _List["dict"]):
    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class RelationshipList(list, _List["dict"]):
    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class CompetitivePriceList(list, _List["CompetitivePriceType"]):
    def __init__(self, data):
        super().__init__([CompetitivePriceType(datum) for datum in data])
        self.data = data


class NumberOfOfferListingsList(list, _List["OfferListingCountType"]):
    def __init__(self, data):
        super().__init__([OfferListingCountType(datum) for datum in data])
        self.data = data


class SalesRankList(list, _List["SalesRankType"]):
    def __init__(self, data):
        super().__init__([SalesRankType(datum) for datum in data])
        self.data = data


class OffersList(list, _List["OfferType"]):
    def __init__(self, data):
        super().__init__([OfferType(datum) for datum in data])
        self.data = data


class BuyBoxEligibleOffers(list, _List["OfferCountType"]):
    def __init__(self, data):
        super().__init__([OfferCountType(datum) for datum in data])
        self.data = data


class BuyBoxPrices(list, _List["BuyBoxPriceType"]):
    def __init__(self, data):
        super().__init__([BuyBoxPriceType(datum) for datum in data])
        self.data = data


class LowestPrices(list, _List["LowestPriceType"]):
    def __init__(self, data):
        super().__init__([LowestPriceType(datum) for datum in data])
        self.data = data


class NumberOfOffers(list, _List["OfferCountType"]):
    def __init__(self, data):
        super().__init__([OfferCountType(datum) for datum in data])
        self.data = data


class OfferDetailList(list, _List["OfferDetail"]):
    def __init__(self, data):
        super().__init__([OfferDetail(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class ConditionType(str):
    pass


class FulfillmentChannelType(str):
    pass


class ProductPricingV0Client(__BaseClient):
    def getPricing(
        self,
        MarketplaceId: str,
        ItemType: str,
        Asins: _List[str] = None,
        Skus: _List[str] = None,
        ItemCondition: str = None,
    ):
        url = "/products/pricing/v0/price".format()
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = (MarketplaceId,)
        if Asins is not None:
            params["Asins"] = ",".join(map(str, Asins))
        if Skus is not None:
            params["Skus"] = ",".join(map(str, Skus))
        if ItemType is not None:
            params["ItemType"] = (ItemType,)
        if ItemCondition is not None:
            params["ItemCondition"] = (ItemCondition,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetPricingResponse,
            400: GetPricingResponse,
            401: GetPricingResponse,
            403: GetPricingResponse,
            404: GetPricingResponse,
            429: GetPricingResponse,
            500: GetPricingResponse,
            503: GetPricingResponse,
        }[response.status_code](response.json())

    def getCompetitivePricing(
        self,
        MarketplaceId: str,
        ItemType: str,
        Asins: _List[str] = None,
        Skus: _List[str] = None,
    ):
        url = "/products/pricing/v0/competitivePrice".format()
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = (MarketplaceId,)
        if Asins is not None:
            params["Asins"] = ",".join(map(str, Asins))
        if Skus is not None:
            params["Skus"] = ",".join(map(str, Skus))
        if ItemType is not None:
            params["ItemType"] = (ItemType,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetPricingResponse,
            400: GetPricingResponse,
            401: GetPricingResponse,
            403: GetPricingResponse,
            404: GetPricingResponse,
            429: GetPricingResponse,
            500: GetPricingResponse,
            503: GetPricingResponse,
        }[response.status_code](response.json())

    def getListingOffers(
        self,
        SellerSKU: str,
        MarketplaceId: str,
        ItemCondition: str,
    ):
        url = "/products/pricing/v0/listings/{SellerSKU}/offers".format(
            SellerSKU=SellerSKU,
        )
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = (MarketplaceId,)
        if ItemCondition is not None:
            params["ItemCondition"] = (ItemCondition,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetOffersResponse,
            400: GetOffersResponse,
            401: GetOffersResponse,
            403: GetOffersResponse,
            404: GetOffersResponse,
            429: GetOffersResponse,
            500: GetOffersResponse,
            503: GetOffersResponse,
        }[response.status_code](response.json())

    def getItemOffers(
        self,
        Asin: str,
        MarketplaceId: str,
        ItemCondition: str,
    ):
        url = "/products/pricing/v0/items/{Asin}/offers".format(
            Asin=Asin,
        )
        params = {}
        if MarketplaceId is not None:
            params["MarketplaceId"] = (MarketplaceId,)
        if ItemCondition is not None:
            params["ItemCondition"] = (ItemCondition,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetOffersResponse,
            400: GetOffersResponse,
            401: GetOffersResponse,
            403: GetOffersResponse,
            404: GetOffersResponse,
            429: GetOffersResponse,
            500: GetOffersResponse,
            503: GetOffersResponse,
        }[response.status_code](response.json())



from .base import BaseClient as __BaseClient
from typing import List as _List


class Error:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "code" in data:
            self.code: str = str(data["code"])
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = str(data["message"])
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = str(data["details"])
        else:
            self.details: str = None


class Address:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "name" in data:
            self.name: str = str(data["name"])
        else:
            self.name: str = None
        if "addressLine1" in data:
            self.addressLine1: str = str(data["addressLine1"])
        else:
            self.addressLine1: str = None
        if "addressLine2" in data:
            self.addressLine2: str = str(data["addressLine2"])
        else:
            self.addressLine2: str = None
        if "addressLine3" in data:
            self.addressLine3: str = str(data["addressLine3"])
        else:
            self.addressLine3: str = None
        if "city" in data:
            self.city: str = str(data["city"])
        else:
            self.city: str = None
        if "districtOrCounty" in data:
            self.districtOrCounty: str = str(data["districtOrCounty"])
        else:
            self.districtOrCounty: str = None
        if "stateOrRegion" in data:
            self.stateOrRegion: str = str(data["stateOrRegion"])
        else:
            self.stateOrRegion: str = None
        if "postalCode" in data:
            self.postalCode: str = str(data["postalCode"])
        else:
            self.postalCode: str = None
        if "countryCode" in data:
            self.countryCode: str = str(data["countryCode"])
        else:
            self.countryCode: str = None
        if "phone" in data:
            self.phone: str = str(data["phone"])
        else:
            self.phone: str = None


class CODSettings:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "isCodRequired" in data:
            self.isCodRequired: bool = bool(data["isCodRequired"])
        else:
            self.isCodRequired: bool = None
        if "codCharge" in data:
            self.codCharge: Money = Money(data["codCharge"])
        else:
            self.codCharge: Money = None
        if "codChargeTax" in data:
            self.codChargeTax: Money = Money(data["codChargeTax"])
        else:
            self.codChargeTax: Money = None
        if "shippingCharge" in data:
            self.shippingCharge: Money = Money(data["shippingCharge"])
        else:
            self.shippingCharge: Money = None
        if "shippingChargeTax" in data:
            self.shippingChargeTax: Money = Money(data["shippingChargeTax"])
        else:
            self.shippingChargeTax: Money = None


class CreateFulfillmentOrderItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "giftMessage" in data:
            self.giftMessage: str = str(data["giftMessage"])
        else:
            self.giftMessage: str = None
        if "displayableComment" in data:
            self.displayableComment: str = str(data["displayableComment"])
        else:
            self.displayableComment: str = None
        if "fulfillmentNetworkSku" in data:
            self.fulfillmentNetworkSku: str = str(data["fulfillmentNetworkSku"])
        else:
            self.fulfillmentNetworkSku: str = None
        if "perUnitDeclaredValue" in data:
            self.perUnitDeclaredValue: Money = Money(data["perUnitDeclaredValue"])
        else:
            self.perUnitDeclaredValue: Money = None
        if "perUnitPrice" in data:
            self.perUnitPrice: Money = Money(data["perUnitPrice"])
        else:
            self.perUnitPrice: Money = None
        if "perUnitTax" in data:
            self.perUnitTax: Money = Money(data["perUnitTax"])
        else:
            self.perUnitTax: Money = None


class CreateFulfillmentOrderRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "sellerFulfillmentOrderId" in data:
            self.sellerFulfillmentOrderId: str = str(data["sellerFulfillmentOrderId"])
        else:
            self.sellerFulfillmentOrderId: str = None
        if "displayableOrderId" in data:
            self.displayableOrderId: str = str(data["displayableOrderId"])
        else:
            self.displayableOrderId: str = None
        if "displayableOrderDate" in data:
            self.displayableOrderDate: Timestamp = Timestamp(data["displayableOrderDate"])
        else:
            self.displayableOrderDate: Timestamp = None
        if "displayableOrderComment" in data:
            self.displayableOrderComment: str = str(data["displayableOrderComment"])
        else:
            self.displayableOrderComment: str = None
        if "shippingSpeedCategory" in data:
            self.shippingSpeedCategory: ShippingSpeedCategory = ShippingSpeedCategory(data["shippingSpeedCategory"])
        else:
            self.shippingSpeedCategory: ShippingSpeedCategory = None
        if "deliveryWindow" in data:
            self.deliveryWindow: DeliveryWindow = DeliveryWindow(data["deliveryWindow"])
        else:
            self.deliveryWindow: DeliveryWindow = None
        if "destinationAddress" in data:
            self.destinationAddress: Address = Address(data["destinationAddress"])
        else:
            self.destinationAddress: Address = None
        if "fulfillmentAction" in data:
            self.fulfillmentAction: FulfillmentAction = FulfillmentAction(data["fulfillmentAction"])
        else:
            self.fulfillmentAction: FulfillmentAction = None
        if "fulfillmentPolicy" in data:
            self.fulfillmentPolicy: FulfillmentPolicy = FulfillmentPolicy(data["fulfillmentPolicy"])
        else:
            self.fulfillmentPolicy: FulfillmentPolicy = None
        if "codSettings" in data:
            self.codSettings: CODSettings = CODSettings(data["codSettings"])
        else:
            self.codSettings: CODSettings = None
        if "shipFromCountryCode" in data:
            self.shipFromCountryCode: str = str(data["shipFromCountryCode"])
        else:
            self.shipFromCountryCode: str = None
        if "notificationEmails" in data:
            self.notificationEmails: NotificationEmailList = NotificationEmailList(data["notificationEmails"])
        else:
            self.notificationEmails: NotificationEmailList = None
        if "featureConstraints" in data:
            self.featureConstraints: _List[FeatureSettings] = [
                FeatureSettings(datum) for datum in data["featureConstraints"]
            ]
        else:
            self.featureConstraints: _List[FeatureSettings] = []
        if "items" in data:
            self.items: CreateFulfillmentOrderItemList = CreateFulfillmentOrderItemList(data["items"])
        else:
            self.items: CreateFulfillmentOrderItemList = None


class CreateFulfillmentReturnRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "items" in data:
            self.items: CreateReturnItemList = CreateReturnItemList(data["items"])
        else:
            self.items: CreateReturnItemList = None


class CreateFulfillmentReturnResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "returnItems" in data:
            self.returnItems: ReturnItemList = ReturnItemList(data["returnItems"])
        else:
            self.returnItems: ReturnItemList = None
        if "invalidReturnItems" in data:
            self.invalidReturnItems: InvalidReturnItemList = InvalidReturnItemList(data["invalidReturnItems"])
        else:
            self.invalidReturnItems: InvalidReturnItemList = None
        if "returnAuthorizations" in data:
            self.returnAuthorizations: ReturnAuthorizationList = ReturnAuthorizationList(data["returnAuthorizations"])
        else:
            self.returnAuthorizations: ReturnAuthorizationList = None


class CreateFulfillmentReturnResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: CreateFulfillmentReturnResult = CreateFulfillmentReturnResult(data["payload"])
        else:
            self.payload: CreateFulfillmentReturnResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class CreateReturnItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerReturnItemId" in data:
            self.sellerReturnItemId: str = str(data["sellerReturnItemId"])
        else:
            self.sellerReturnItemId: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "amazonShipmentId" in data:
            self.amazonShipmentId: str = str(data["amazonShipmentId"])
        else:
            self.amazonShipmentId: str = None
        if "returnReasonCode" in data:
            self.returnReasonCode: str = str(data["returnReasonCode"])
        else:
            self.returnReasonCode: str = None
        if "returnComment" in data:
            self.returnComment: str = str(data["returnComment"])
        else:
            self.returnComment: str = None


class Money:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "currencyCode" in data:
            self.currencyCode: str = str(data["currencyCode"])
        else:
            self.currencyCode: str = None
        if "value" in data:
            self.value: Decimal = Decimal(data["value"])
        else:
            self.value: Decimal = None


class DeliveryWindow:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "startDate" in data:
            self.startDate: Timestamp = Timestamp(data["startDate"])
        else:
            self.startDate: Timestamp = None
        if "endDate" in data:
            self.endDate: Timestamp = Timestamp(data["endDate"])
        else:
            self.endDate: Timestamp = None


class Fee:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "name" in data:
            self.name: str = str(data["name"])
        else:
            self.name: str = None
        if "amount" in data:
            self.amount: Money = Money(data["amount"])
        else:
            self.amount: Money = None


class FulfillmentOrder:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerFulfillmentOrderId" in data:
            self.sellerFulfillmentOrderId: str = str(data["sellerFulfillmentOrderId"])
        else:
            self.sellerFulfillmentOrderId: str = None
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "displayableOrderId" in data:
            self.displayableOrderId: str = str(data["displayableOrderId"])
        else:
            self.displayableOrderId: str = None
        if "displayableOrderDate" in data:
            self.displayableOrderDate: Timestamp = Timestamp(data["displayableOrderDate"])
        else:
            self.displayableOrderDate: Timestamp = None
        if "displayableOrderComment" in data:
            self.displayableOrderComment: str = str(data["displayableOrderComment"])
        else:
            self.displayableOrderComment: str = None
        if "shippingSpeedCategory" in data:
            self.shippingSpeedCategory: ShippingSpeedCategory = ShippingSpeedCategory(data["shippingSpeedCategory"])
        else:
            self.shippingSpeedCategory: ShippingSpeedCategory = None
        if "deliveryWindow" in data:
            self.deliveryWindow: DeliveryWindow = DeliveryWindow(data["deliveryWindow"])
        else:
            self.deliveryWindow: DeliveryWindow = None
        if "destinationAddress" in data:
            self.destinationAddress: Address = Address(data["destinationAddress"])
        else:
            self.destinationAddress: Address = None
        if "fulfillmentAction" in data:
            self.fulfillmentAction: FulfillmentAction = FulfillmentAction(data["fulfillmentAction"])
        else:
            self.fulfillmentAction: FulfillmentAction = None
        if "fulfillmentPolicy" in data:
            self.fulfillmentPolicy: FulfillmentPolicy = FulfillmentPolicy(data["fulfillmentPolicy"])
        else:
            self.fulfillmentPolicy: FulfillmentPolicy = None
        if "codSettings" in data:
            self.codSettings: CODSettings = CODSettings(data["codSettings"])
        else:
            self.codSettings: CODSettings = None
        if "receivedDate" in data:
            self.receivedDate: Timestamp = Timestamp(data["receivedDate"])
        else:
            self.receivedDate: Timestamp = None
        if "fulfillmentOrderStatus" in data:
            self.fulfillmentOrderStatus: FulfillmentOrderStatus = FulfillmentOrderStatus(data["fulfillmentOrderStatus"])
        else:
            self.fulfillmentOrderStatus: FulfillmentOrderStatus = None
        if "statusUpdatedDate" in data:
            self.statusUpdatedDate: Timestamp = Timestamp(data["statusUpdatedDate"])
        else:
            self.statusUpdatedDate: Timestamp = None
        if "notificationEmails" in data:
            self.notificationEmails: NotificationEmailList = NotificationEmailList(data["notificationEmails"])
        else:
            self.notificationEmails: NotificationEmailList = None
        if "featureConstraints" in data:
            self.featureConstraints: _List[FeatureSettings] = [
                FeatureSettings(datum) for datum in data["featureConstraints"]
            ]
        else:
            self.featureConstraints: _List[FeatureSettings] = []


class FulfillmentOrderItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "giftMessage" in data:
            self.giftMessage: str = str(data["giftMessage"])
        else:
            self.giftMessage: str = None
        if "displayableComment" in data:
            self.displayableComment: str = str(data["displayableComment"])
        else:
            self.displayableComment: str = None
        if "fulfillmentNetworkSku" in data:
            self.fulfillmentNetworkSku: str = str(data["fulfillmentNetworkSku"])
        else:
            self.fulfillmentNetworkSku: str = None
        if "orderItemDisposition" in data:
            self.orderItemDisposition: str = str(data["orderItemDisposition"])
        else:
            self.orderItemDisposition: str = None
        if "cancelledQuantity" in data:
            self.cancelledQuantity: Quantity = Quantity(data["cancelledQuantity"])
        else:
            self.cancelledQuantity: Quantity = None
        if "unfulfillableQuantity" in data:
            self.unfulfillableQuantity: Quantity = Quantity(data["unfulfillableQuantity"])
        else:
            self.unfulfillableQuantity: Quantity = None
        if "estimatedShipDate" in data:
            self.estimatedShipDate: Timestamp = Timestamp(data["estimatedShipDate"])
        else:
            self.estimatedShipDate: Timestamp = None
        if "estimatedArrivalDate" in data:
            self.estimatedArrivalDate: Timestamp = Timestamp(data["estimatedArrivalDate"])
        else:
            self.estimatedArrivalDate: Timestamp = None
        if "perUnitPrice" in data:
            self.perUnitPrice: Money = Money(data["perUnitPrice"])
        else:
            self.perUnitPrice: Money = None
        if "perUnitTax" in data:
            self.perUnitTax: Money = Money(data["perUnitTax"])
        else:
            self.perUnitTax: Money = None
        if "perUnitDeclaredValue" in data:
            self.perUnitDeclaredValue: Money = Money(data["perUnitDeclaredValue"])
        else:
            self.perUnitDeclaredValue: Money = None


class FulfillmentPreview:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "shippingSpeedCategory" in data:
            self.shippingSpeedCategory: ShippingSpeedCategory = ShippingSpeedCategory(data["shippingSpeedCategory"])
        else:
            self.shippingSpeedCategory: ShippingSpeedCategory = None
        if "scheduledDeliveryInfo" in data:
            self.scheduledDeliveryInfo: ScheduledDeliveryInfo = ScheduledDeliveryInfo(data["scheduledDeliveryInfo"])
        else:
            self.scheduledDeliveryInfo: ScheduledDeliveryInfo = None
        if "isFulfillable" in data:
            self.isFulfillable: bool = bool(data["isFulfillable"])
        else:
            self.isFulfillable: bool = None
        if "isCODCapable" in data:
            self.isCODCapable: bool = bool(data["isCODCapable"])
        else:
            self.isCODCapable: bool = None
        if "estimatedShippingWeight" in data:
            self.estimatedShippingWeight: Weight = Weight(data["estimatedShippingWeight"])
        else:
            self.estimatedShippingWeight: Weight = None
        if "estimatedFees" in data:
            self.estimatedFees: FeeList = FeeList(data["estimatedFees"])
        else:
            self.estimatedFees: FeeList = None
        if "fulfillmentPreviewShipments" in data:
            self.fulfillmentPreviewShipments: FulfillmentPreviewShipmentList = FulfillmentPreviewShipmentList(
                data["fulfillmentPreviewShipments"]
            )
        else:
            self.fulfillmentPreviewShipments: FulfillmentPreviewShipmentList = None
        if "unfulfillablePreviewItems" in data:
            self.unfulfillablePreviewItems: UnfulfillablePreviewItemList = UnfulfillablePreviewItemList(
                data["unfulfillablePreviewItems"]
            )
        else:
            self.unfulfillablePreviewItems: UnfulfillablePreviewItemList = None
        if "orderUnfulfillableReasons" in data:
            self.orderUnfulfillableReasons: StringList = StringList(data["orderUnfulfillableReasons"])
        else:
            self.orderUnfulfillableReasons: StringList = None
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "featureConstraints" in data:
            self.featureConstraints: _List[FeatureSettings] = [
                FeatureSettings(datum) for datum in data["featureConstraints"]
            ]
        else:
            self.featureConstraints: _List[FeatureSettings] = []


class FulfillmentPreviewItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "estimatedShippingWeight" in data:
            self.estimatedShippingWeight: Weight = Weight(data["estimatedShippingWeight"])
        else:
            self.estimatedShippingWeight: Weight = None
        if "shippingWeightCalculationMethod" in data:
            self.shippingWeightCalculationMethod: str = str(data["shippingWeightCalculationMethod"])
        else:
            self.shippingWeightCalculationMethod: str = None


class FulfillmentPreviewShipment:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "earliestShipDate" in data:
            self.earliestShipDate: Timestamp = Timestamp(data["earliestShipDate"])
        else:
            self.earliestShipDate: Timestamp = None
        if "latestShipDate" in data:
            self.latestShipDate: Timestamp = Timestamp(data["latestShipDate"])
        else:
            self.latestShipDate: Timestamp = None
        if "earliestArrivalDate" in data:
            self.earliestArrivalDate: Timestamp = Timestamp(data["earliestArrivalDate"])
        else:
            self.earliestArrivalDate: Timestamp = None
        if "latestArrivalDate" in data:
            self.latestArrivalDate: Timestamp = Timestamp(data["latestArrivalDate"])
        else:
            self.latestArrivalDate: Timestamp = None
        if "shippingNotes" in data:
            self.shippingNotes: _List[str] = [str(datum) for datum in data["shippingNotes"]]
        else:
            self.shippingNotes: _List[str] = []
        if "fulfillmentPreviewItems" in data:
            self.fulfillmentPreviewItems: FulfillmentPreviewItemList = FulfillmentPreviewItemList(
                data["fulfillmentPreviewItems"]
            )
        else:
            self.fulfillmentPreviewItems: FulfillmentPreviewItemList = None


class FulfillmentShipment:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "amazonShipmentId" in data:
            self.amazonShipmentId: str = str(data["amazonShipmentId"])
        else:
            self.amazonShipmentId: str = None
        if "fulfillmentCenterId" in data:
            self.fulfillmentCenterId: str = str(data["fulfillmentCenterId"])
        else:
            self.fulfillmentCenterId: str = None
        if "fulfillmentShipmentStatus" in data:
            self.fulfillmentShipmentStatus: str = str(data["fulfillmentShipmentStatus"])
        else:
            self.fulfillmentShipmentStatus: str = None
        if "shippingDate" in data:
            self.shippingDate: Timestamp = Timestamp(data["shippingDate"])
        else:
            self.shippingDate: Timestamp = None
        if "estimatedArrivalDate" in data:
            self.estimatedArrivalDate: Timestamp = Timestamp(data["estimatedArrivalDate"])
        else:
            self.estimatedArrivalDate: Timestamp = None
        if "shippingNotes" in data:
            self.shippingNotes: _List[str] = [str(datum) for datum in data["shippingNotes"]]
        else:
            self.shippingNotes: _List[str] = []
        if "fulfillmentShipmentItem" in data:
            self.fulfillmentShipmentItem: FulfillmentShipmentItemList = FulfillmentShipmentItemList(
                data["fulfillmentShipmentItem"]
            )
        else:
            self.fulfillmentShipmentItem: FulfillmentShipmentItemList = None
        if "fulfillmentShipmentPackage" in data:
            self.fulfillmentShipmentPackage: FulfillmentShipmentPackageList = FulfillmentShipmentPackageList(
                data["fulfillmentShipmentPackage"]
            )
        else:
            self.fulfillmentShipmentPackage: FulfillmentShipmentPackageList = None


class FulfillmentShipmentItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "packageNumber" in data:
            self.packageNumber: int = int(data["packageNumber"])
        else:
            self.packageNumber: int = None
        if "serialNumber" in data:
            self.serialNumber: str = str(data["serialNumber"])
        else:
            self.serialNumber: str = None


class FulfillmentShipmentPackage:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "packageNumber" in data:
            self.packageNumber: int = int(data["packageNumber"])
        else:
            self.packageNumber: int = None
        if "carrierCode" in data:
            self.carrierCode: str = str(data["carrierCode"])
        else:
            self.carrierCode: str = None
        if "trackingNumber" in data:
            self.trackingNumber: str = str(data["trackingNumber"])
        else:
            self.trackingNumber: str = None
        if "estimatedArrivalDate" in data:
            self.estimatedArrivalDate: Timestamp = Timestamp(data["estimatedArrivalDate"])
        else:
            self.estimatedArrivalDate: Timestamp = None


class GetFulfillmentOrderResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "fulfillmentOrder" in data:
            self.fulfillmentOrder: FulfillmentOrder = FulfillmentOrder(data["fulfillmentOrder"])
        else:
            self.fulfillmentOrder: FulfillmentOrder = None
        if "fulfillmentOrderItems" in data:
            self.fulfillmentOrderItems: FulfillmentOrderItemList = FulfillmentOrderItemList(
                data["fulfillmentOrderItems"]
            )
        else:
            self.fulfillmentOrderItems: FulfillmentOrderItemList = None
        if "fulfillmentShipments" in data:
            self.fulfillmentShipments: FulfillmentShipmentList = FulfillmentShipmentList(data["fulfillmentShipments"])
        else:
            self.fulfillmentShipments: FulfillmentShipmentList = None
        if "returnItems" in data:
            self.returnItems: ReturnItemList = ReturnItemList(data["returnItems"])
        else:
            self.returnItems: ReturnItemList = None
        if "returnAuthorizations" in data:
            self.returnAuthorizations: ReturnAuthorizationList = ReturnAuthorizationList(data["returnAuthorizations"])
        else:
            self.returnAuthorizations: ReturnAuthorizationList = None


class GetFulfillmentOrderResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetFulfillmentOrderResult = GetFulfillmentOrderResult(data["payload"])
        else:
            self.payload: GetFulfillmentOrderResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetFulfillmentPreviewItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "perUnitDeclaredValue" in data:
            self.perUnitDeclaredValue: Money = Money(data["perUnitDeclaredValue"])
        else:
            self.perUnitDeclaredValue: Money = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None


class GetFulfillmentPreviewRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "address" in data:
            self.address: Address = Address(data["address"])
        else:
            self.address: Address = None
        if "items" in data:
            self.items: GetFulfillmentPreviewItemList = GetFulfillmentPreviewItemList(data["items"])
        else:
            self.items: GetFulfillmentPreviewItemList = None
        if "shippingSpeedCategories" in data:
            self.shippingSpeedCategories: ShippingSpeedCategoryList = ShippingSpeedCategoryList(
                data["shippingSpeedCategories"]
            )
        else:
            self.shippingSpeedCategories: ShippingSpeedCategoryList = None
        if "includeCODFulfillmentPreview" in data:
            self.includeCODFulfillmentPreview: bool = bool(data["includeCODFulfillmentPreview"])
        else:
            self.includeCODFulfillmentPreview: bool = None
        if "includeDeliveryWindows" in data:
            self.includeDeliveryWindows: bool = bool(data["includeDeliveryWindows"])
        else:
            self.includeDeliveryWindows: bool = None
        if "featureConstraints" in data:
            self.featureConstraints: _List[FeatureSettings] = [
                FeatureSettings(datum) for datum in data["featureConstraints"]
            ]
        else:
            self.featureConstraints: _List[FeatureSettings] = []


class GetFulfillmentPreviewResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "fulfillmentPreviews" in data:
            self.fulfillmentPreviews: FulfillmentPreviewList = FulfillmentPreviewList(data["fulfillmentPreviews"])
        else:
            self.fulfillmentPreviews: FulfillmentPreviewList = None


class GetFulfillmentPreviewResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetFulfillmentPreviewResult = GetFulfillmentPreviewResult(data["payload"])
        else:
            self.payload: GetFulfillmentPreviewResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class InvalidItemReason:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "invalidItemReasonCode" in data:
            self.invalidItemReasonCode: InvalidItemReasonCode = InvalidItemReasonCode(data["invalidItemReasonCode"])
        else:
            self.invalidItemReasonCode: InvalidItemReasonCode = None
        if "description" in data:
            self.description: str = str(data["description"])
        else:
            self.description: str = None


class InvalidReturnItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerReturnItemId" in data:
            self.sellerReturnItemId: str = str(data["sellerReturnItemId"])
        else:
            self.sellerReturnItemId: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "invalidItemReason" in data:
            self.invalidItemReason: InvalidItemReason = InvalidItemReason(data["invalidItemReason"])
        else:
            self.invalidItemReason: InvalidItemReason = None


class ListAllFulfillmentOrdersResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "nextToken" in data:
            self.nextToken: str = str(data["nextToken"])
        else:
            self.nextToken: str = None
        if "fulfillmentOrders" in data:
            self.fulfillmentOrders: _List[FulfillmentOrder] = [
                FulfillmentOrder(datum) for datum in data["fulfillmentOrders"]
            ]
        else:
            self.fulfillmentOrders: _List[FulfillmentOrder] = []


class ListAllFulfillmentOrdersResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: ListAllFulfillmentOrdersResult = ListAllFulfillmentOrdersResult(data["payload"])
        else:
            self.payload: ListAllFulfillmentOrdersResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class ListReturnReasonCodesResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "reasonCodeDetails" in data:
            self.reasonCodeDetails: ReasonCodeDetailsList = ReasonCodeDetailsList(data["reasonCodeDetails"])
        else:
            self.reasonCodeDetails: ReasonCodeDetailsList = None


class ListReturnReasonCodesResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: ListReturnReasonCodesResult = ListReturnReasonCodesResult(data["payload"])
        else:
            self.payload: ListReturnReasonCodesResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class PackageTrackingDetails:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "packageNumber" in data:
            self.packageNumber: int = int(data["packageNumber"])
        else:
            self.packageNumber: int = None
        if "trackingNumber" in data:
            self.trackingNumber: str = str(data["trackingNumber"])
        else:
            self.trackingNumber: str = None
        if "customerTrackingLink" in data:
            self.customerTrackingLink: str = str(data["customerTrackingLink"])
        else:
            self.customerTrackingLink: str = None
        if "carrierCode" in data:
            self.carrierCode: str = str(data["carrierCode"])
        else:
            self.carrierCode: str = None
        if "carrierPhoneNumber" in data:
            self.carrierPhoneNumber: str = str(data["carrierPhoneNumber"])
        else:
            self.carrierPhoneNumber: str = None
        if "carrierURL" in data:
            self.carrierURL: str = str(data["carrierURL"])
        else:
            self.carrierURL: str = None
        if "shipDate" in data:
            self.shipDate: Timestamp = Timestamp(data["shipDate"])
        else:
            self.shipDate: Timestamp = None
        if "estimatedArrivalDate" in data:
            self.estimatedArrivalDate: Timestamp = Timestamp(data["estimatedArrivalDate"])
        else:
            self.estimatedArrivalDate: Timestamp = None
        if "shipToAddress" in data:
            self.shipToAddress: TrackingAddress = TrackingAddress(data["shipToAddress"])
        else:
            self.shipToAddress: TrackingAddress = None
        if "currentStatus" in data:
            self.currentStatus: CurrentStatus = CurrentStatus(data["currentStatus"])
        else:
            self.currentStatus: CurrentStatus = None
        if "currentStatusDescription" in data:
            self.currentStatusDescription: str = str(data["currentStatusDescription"])
        else:
            self.currentStatusDescription: str = None
        if "signedForBy" in data:
            self.signedForBy: str = str(data["signedForBy"])
        else:
            self.signedForBy: str = None
        if "additionalLocationInfo" in data:
            self.additionalLocationInfo: AdditionalLocationInfo = AdditionalLocationInfo(data["additionalLocationInfo"])
        else:
            self.additionalLocationInfo: AdditionalLocationInfo = None
        if "trackingEvents" in data:
            self.trackingEvents: TrackingEventList = TrackingEventList(data["trackingEvents"])
        else:
            self.trackingEvents: TrackingEventList = None


class GetPackageTrackingDetailsResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: PackageTrackingDetails = PackageTrackingDetails(data["payload"])
        else:
            self.payload: PackageTrackingDetails = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class ReasonCodeDetails:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "returnReasonCode" in data:
            self.returnReasonCode: str = str(data["returnReasonCode"])
        else:
            self.returnReasonCode: str = None
        if "description" in data:
            self.description: str = str(data["description"])
        else:
            self.description: str = None
        if "translatedDescription" in data:
            self.translatedDescription: str = str(data["translatedDescription"])
        else:
            self.translatedDescription: str = None


class ReturnAuthorization:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "returnAuthorizationId" in data:
            self.returnAuthorizationId: str = str(data["returnAuthorizationId"])
        else:
            self.returnAuthorizationId: str = None
        if "fulfillmentCenterId" in data:
            self.fulfillmentCenterId: str = str(data["fulfillmentCenterId"])
        else:
            self.fulfillmentCenterId: str = None
        if "returnToAddress" in data:
            self.returnToAddress: Address = Address(data["returnToAddress"])
        else:
            self.returnToAddress: Address = None
        if "amazonRmaId" in data:
            self.amazonRmaId: str = str(data["amazonRmaId"])
        else:
            self.amazonRmaId: str = None
        if "rmaPageURL" in data:
            self.rmaPageURL: str = str(data["rmaPageURL"])
        else:
            self.rmaPageURL: str = None


class ReturnItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerReturnItemId" in data:
            self.sellerReturnItemId: str = str(data["sellerReturnItemId"])
        else:
            self.sellerReturnItemId: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "amazonShipmentId" in data:
            self.amazonShipmentId: str = str(data["amazonShipmentId"])
        else:
            self.amazonShipmentId: str = None
        if "sellerReturnReasonCode" in data:
            self.sellerReturnReasonCode: str = str(data["sellerReturnReasonCode"])
        else:
            self.sellerReturnReasonCode: str = None
        if "returnComment" in data:
            self.returnComment: str = str(data["returnComment"])
        else:
            self.returnComment: str = None
        if "amazonReturnReasonCode" in data:
            self.amazonReturnReasonCode: str = str(data["amazonReturnReasonCode"])
        else:
            self.amazonReturnReasonCode: str = None
        if "status" in data:
            self.status: FulfillmentReturnItemStatus = FulfillmentReturnItemStatus(data["status"])
        else:
            self.status: FulfillmentReturnItemStatus = None
        if "statusChangedDate" in data:
            self.statusChangedDate: Timestamp = Timestamp(data["statusChangedDate"])
        else:
            self.statusChangedDate: Timestamp = None
        if "returnAuthorizationId" in data:
            self.returnAuthorizationId: str = str(data["returnAuthorizationId"])
        else:
            self.returnAuthorizationId: str = None
        if "returnReceivedCondition" in data:
            self.returnReceivedCondition: ReturnItemDisposition = ReturnItemDisposition(data["returnReceivedCondition"])
        else:
            self.returnReceivedCondition: ReturnItemDisposition = None
        if "fulfillmentCenterId" in data:
            self.fulfillmentCenterId: str = str(data["fulfillmentCenterId"])
        else:
            self.fulfillmentCenterId: str = None


class ScheduledDeliveryInfo:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "deliveryTimeZone" in data:
            self.deliveryTimeZone: str = str(data["deliveryTimeZone"])
        else:
            self.deliveryTimeZone: str = None
        if "deliveryWindows" in data:
            self.deliveryWindows: DeliveryWindowList = DeliveryWindowList(data["deliveryWindows"])
        else:
            self.deliveryWindows: DeliveryWindowList = None


class TrackingAddress:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "city" in data:
            self.city: str = str(data["city"])
        else:
            self.city: str = None
        if "state" in data:
            self.state: str = str(data["state"])
        else:
            self.state: str = None
        if "country" in data:
            self.country: str = str(data["country"])
        else:
            self.country: str = None


class TrackingEvent:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "eventDate" in data:
            self.eventDate: Timestamp = Timestamp(data["eventDate"])
        else:
            self.eventDate: Timestamp = None
        if "eventAddress" in data:
            self.eventAddress: TrackingAddress = TrackingAddress(data["eventAddress"])
        else:
            self.eventAddress: TrackingAddress = None
        if "eventCode" in data:
            self.eventCode: EventCode = EventCode(data["eventCode"])
        else:
            self.eventCode: EventCode = None
        if "eventDescription" in data:
            self.eventDescription: str = str(data["eventDescription"])
        else:
            self.eventDescription: str = None


class UnfulfillablePreviewItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "itemUnfulfillableReasons" in data:
            self.itemUnfulfillableReasons: StringList = StringList(data["itemUnfulfillableReasons"])
        else:
            self.itemUnfulfillableReasons: StringList = None


class UpdateFulfillmentOrderItem:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "sellerFulfillmentOrderItemId" in data:
            self.sellerFulfillmentOrderItemId: str = str(data["sellerFulfillmentOrderItemId"])
        else:
            self.sellerFulfillmentOrderItemId: str = None
        if "quantity" in data:
            self.quantity: Quantity = Quantity(data["quantity"])
        else:
            self.quantity: Quantity = None
        if "giftMessage" in data:
            self.giftMessage: str = str(data["giftMessage"])
        else:
            self.giftMessage: str = None
        if "displayableComment" in data:
            self.displayableComment: str = str(data["displayableComment"])
        else:
            self.displayableComment: str = None
        if "fulfillmentNetworkSku" in data:
            self.fulfillmentNetworkSku: str = str(data["fulfillmentNetworkSku"])
        else:
            self.fulfillmentNetworkSku: str = None
        if "orderItemDisposition" in data:
            self.orderItemDisposition: str = str(data["orderItemDisposition"])
        else:
            self.orderItemDisposition: str = None
        if "perUnitDeclaredValue" in data:
            self.perUnitDeclaredValue: Money = Money(data["perUnitDeclaredValue"])
        else:
            self.perUnitDeclaredValue: Money = None
        if "perUnitPrice" in data:
            self.perUnitPrice: Money = Money(data["perUnitPrice"])
        else:
            self.perUnitPrice: Money = None
        if "perUnitTax" in data:
            self.perUnitTax: Money = Money(data["perUnitTax"])
        else:
            self.perUnitTax: Money = None


class UpdateFulfillmentOrderRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "displayableOrderId" in data:
            self.displayableOrderId: str = str(data["displayableOrderId"])
        else:
            self.displayableOrderId: str = None
        if "displayableOrderDate" in data:
            self.displayableOrderDate: Timestamp = Timestamp(data["displayableOrderDate"])
        else:
            self.displayableOrderDate: Timestamp = None
        if "displayableOrderComment" in data:
            self.displayableOrderComment: str = str(data["displayableOrderComment"])
        else:
            self.displayableOrderComment: str = None
        if "shippingSpeedCategory" in data:
            self.shippingSpeedCategory: ShippingSpeedCategory = ShippingSpeedCategory(data["shippingSpeedCategory"])
        else:
            self.shippingSpeedCategory: ShippingSpeedCategory = None
        if "destinationAddress" in data:
            self.destinationAddress: Address = Address(data["destinationAddress"])
        else:
            self.destinationAddress: Address = None
        if "fulfillmentAction" in data:
            self.fulfillmentAction: FulfillmentAction = FulfillmentAction(data["fulfillmentAction"])
        else:
            self.fulfillmentAction: FulfillmentAction = None
        if "fulfillmentPolicy" in data:
            self.fulfillmentPolicy: FulfillmentPolicy = FulfillmentPolicy(data["fulfillmentPolicy"])
        else:
            self.fulfillmentPolicy: FulfillmentPolicy = None
        if "shipFromCountryCode" in data:
            self.shipFromCountryCode: str = str(data["shipFromCountryCode"])
        else:
            self.shipFromCountryCode: str = None
        if "notificationEmails" in data:
            self.notificationEmails: NotificationEmailList = NotificationEmailList(data["notificationEmails"])
        else:
            self.notificationEmails: NotificationEmailList = None
        if "featureConstraints" in data:
            self.featureConstraints: _List[FeatureSettings] = [
                FeatureSettings(datum) for datum in data["featureConstraints"]
            ]
        else:
            self.featureConstraints: _List[FeatureSettings] = []
        if "items" in data:
            self.items: UpdateFulfillmentOrderItemList = UpdateFulfillmentOrderItemList(data["items"])
        else:
            self.items: UpdateFulfillmentOrderItemList = None


class UpdateFulfillmentOrderResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class CreateFulfillmentOrderResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class CancelFulfillmentOrderResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class Weight:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "unit" in data:
            self.unit: str = str(data["unit"])
        else:
            self.unit: str = None
        if "value" in data:
            self.value: str = str(data["value"])
        else:
            self.value: str = None


class GetFeatureInventoryResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetFeatureInventoryResult = GetFeatureInventoryResult(data["payload"])
        else:
            self.payload: GetFeatureInventoryResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetFeatureInventoryResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "featureName" in data:
            self.featureName: str = str(data["featureName"])
        else:
            self.featureName: str = None
        if "nextToken" in data:
            self.nextToken: str = str(data["nextToken"])
        else:
            self.nextToken: str = None
        if "featureSkus" in data:
            self.featureSkus: _List[FeatureSku] = [FeatureSku(datum) for datum in data["featureSkus"]]
        else:
            self.featureSkus: _List[FeatureSku] = []


class FeatureSku:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "fnSku" in data:
            self.fnSku: str = str(data["fnSku"])
        else:
            self.fnSku: str = None
        if "asin" in data:
            self.asin: str = str(data["asin"])
        else:
            self.asin: str = None
        if "skuCount" in data:
            self.skuCount: float = float(data["skuCount"])
        else:
            self.skuCount: float = None
        if "overlappingSkus" in data:
            self.overlappingSkus: _List[str] = [str(datum) for datum in data["overlappingSkus"]]
        else:
            self.overlappingSkus: _List[str] = []


class GetFeaturesResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetFeaturesResult = GetFeaturesResult(data["payload"])
        else:
            self.payload: GetFeaturesResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetFeaturesResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "features" in data:
            self.features: Features = Features(data["features"])
        else:
            self.features: Features = None


class Feature:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "featureName" in data:
            self.featureName: str = str(data["featureName"])
        else:
            self.featureName: str = None
        if "featureDescription" in data:
            self.featureDescription: str = str(data["featureDescription"])
        else:
            self.featureDescription: str = None
        if "sellerEligible" in data:
            self.sellerEligible: bool = bool(data["sellerEligible"])
        else:
            self.sellerEligible: bool = None


class GetFeatureSkuResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetFeatureSkuResult = GetFeatureSkuResult(data["payload"])
        else:
            self.payload: GetFeatureSkuResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetFeatureSkuResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "featureName" in data:
            self.featureName: str = str(data["featureName"])
        else:
            self.featureName: str = None
        if "isEligible" in data:
            self.isEligible: bool = bool(data["isEligible"])
        else:
            self.isEligible: bool = None
        if "ineligibleReasons" in data:
            self.ineligibleReasons: _List[str] = [str(datum) for datum in data["ineligibleReasons"]]
        else:
            self.ineligibleReasons: _List[str] = []
        if "skuInfo" in data:
            self.skuInfo: FeatureSku = FeatureSku(data["skuInfo"])
        else:
            self.skuInfo: FeatureSku = None


class FeatureSettings:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "featureName" in data:
            self.featureName: str = str(data["featureName"])
        else:
            self.featureName: str = None
        if "featureFulfillmentPolicy" in data:
            self.featureFulfillmentPolicy: str = str(data["featureFulfillmentPolicy"])
        else:
            self.featureFulfillmentPolicy: str = None


class ErrorList(list, _List["Error"]):
    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class CreateFulfillmentOrderItemList(list, _List["CreateFulfillmentOrderItem"]):
    def __init__(self, data):
        super().__init__([CreateFulfillmentOrderItem(datum) for datum in data])
        self.data = data


class CreateReturnItemList(list, _List["CreateReturnItem"]):
    def __init__(self, data):
        super().__init__([CreateReturnItem(datum) for datum in data])
        self.data = data


class DeliveryWindowList(list, _List["DeliveryWindow"]):
    def __init__(self, data):
        super().__init__([DeliveryWindow(datum) for datum in data])
        self.data = data


class FeeList(list, _List["Fee"]):
    def __init__(self, data):
        super().__init__([Fee(datum) for datum in data])
        self.data = data


class FulfillmentOrderItemList(list, _List["FulfillmentOrderItem"]):
    def __init__(self, data):
        super().__init__([FulfillmentOrderItem(datum) for datum in data])
        self.data = data


class FulfillmentPreviewItemList(list, _List["FulfillmentPreviewItem"]):
    def __init__(self, data):
        super().__init__([FulfillmentPreviewItem(datum) for datum in data])
        self.data = data


class FulfillmentPreviewList(list, _List["FulfillmentPreview"]):
    def __init__(self, data):
        super().__init__([FulfillmentPreview(datum) for datum in data])
        self.data = data


class FulfillmentPreviewShipmentList(list, _List["FulfillmentPreviewShipment"]):
    def __init__(self, data):
        super().__init__([FulfillmentPreviewShipment(datum) for datum in data])
        self.data = data


class FulfillmentShipmentItemList(list, _List["FulfillmentShipmentItem"]):
    def __init__(self, data):
        super().__init__([FulfillmentShipmentItem(datum) for datum in data])
        self.data = data


class FulfillmentShipmentList(list, _List["FulfillmentShipment"]):
    def __init__(self, data):
        super().__init__([FulfillmentShipment(datum) for datum in data])
        self.data = data


class FulfillmentShipmentPackageList(list, _List["FulfillmentShipmentPackage"]):
    def __init__(self, data):
        super().__init__([FulfillmentShipmentPackage(datum) for datum in data])
        self.data = data


class GetFulfillmentPreviewItemList(list, _List["GetFulfillmentPreviewItem"]):
    def __init__(self, data):
        super().__init__([GetFulfillmentPreviewItem(datum) for datum in data])
        self.data = data


class InvalidReturnItemList(list, _List["InvalidReturnItem"]):
    def __init__(self, data):
        super().__init__([InvalidReturnItem(datum) for datum in data])
        self.data = data


class NotificationEmailList(list, _List["str"]):
    def __init__(self, data):
        super().__init__([str(datum) for datum in data])
        self.data = data


class ReasonCodeDetailsList(list, _List["ReasonCodeDetails"]):
    def __init__(self, data):
        super().__init__([ReasonCodeDetails(datum) for datum in data])
        self.data = data


class ReturnAuthorizationList(list, _List["ReturnAuthorization"]):
    def __init__(self, data):
        super().__init__([ReturnAuthorization(datum) for datum in data])
        self.data = data


class ReturnItemList(list, _List["ReturnItem"]):
    def __init__(self, data):
        super().__init__([ReturnItem(datum) for datum in data])
        self.data = data


class ShippingSpeedCategoryList(list, _List["ShippingSpeedCategory"]):
    def __init__(self, data):
        super().__init__([ShippingSpeedCategory(datum) for datum in data])
        self.data = data


class StringList(list, _List["str"]):
    def __init__(self, data):
        super().__init__([str(datum) for datum in data])
        self.data = data


class TrackingEventList(list, _List["TrackingEvent"]):
    def __init__(self, data):
        super().__init__([TrackingEvent(datum) for datum in data])
        self.data = data


class UnfulfillablePreviewItemList(list, _List["UnfulfillablePreviewItem"]):
    def __init__(self, data):
        super().__init__([UnfulfillablePreviewItem(datum) for datum in data])
        self.data = data


class UpdateFulfillmentOrderItemList(list, _List["UpdateFulfillmentOrderItem"]):
    def __init__(self, data):
        super().__init__([UpdateFulfillmentOrderItem(datum) for datum in data])
        self.data = data


class Features(list, _List["Feature"]):
    def __init__(self, data):
        super().__init__([Feature(datum) for datum in data])
        self.data = data


class FulfillmentPolicy(str):
    pass


class FulfillmentOrderStatus(str):
    pass


class Decimal(str):
    pass


class FulfillmentAction(str):
    pass


class FulfillmentReturnItemStatus(str):
    pass


class InvalidItemReasonCode(str):
    pass


class CurrentStatus(str):
    pass


class AdditionalLocationInfo(str):
    pass


class ReturnItemDisposition(str):
    pass


class Timestamp(str):
    pass


class EventCode(str):
    pass


class Quantity(int):
    pass


class ShippingSpeedCategory(str):
    pass


class FulfillmentOutbound20200701Client(__BaseClient):
    def getFulfillmentPreview(
        self,
        data: GetFulfillmentPreviewRequest,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders/preview".format()
        params = {}
        response = self.request(url, method="POST", data=data.data)
        return {
            200: GetFulfillmentPreviewResponse,
            400: GetFulfillmentPreviewResponse,
            401: GetFulfillmentPreviewResponse,
            403: GetFulfillmentPreviewResponse,
            404: GetFulfillmentPreviewResponse,
            429: GetFulfillmentPreviewResponse,
            500: GetFulfillmentPreviewResponse,
            503: GetFulfillmentPreviewResponse,
        }[response.status_code](response.json())

    def listAllFulfillmentOrders(
        self,
        queryStartDate: str = None,
        nextToken: str = None,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders".format()
        params = {}
        if queryStartDate is not None:
            params["queryStartDate"] = (queryStartDate,)
        if nextToken is not None:
            params["nextToken"] = (nextToken,)
        response = self.request(url, method="GET", params=params)
        return {
            200: ListAllFulfillmentOrdersResponse,
            400: ListAllFulfillmentOrdersResponse,
            401: ListAllFulfillmentOrdersResponse,
            403: ListAllFulfillmentOrdersResponse,
            404: ListAllFulfillmentOrdersResponse,
            429: ListAllFulfillmentOrdersResponse,
            500: ListAllFulfillmentOrdersResponse,
            503: ListAllFulfillmentOrdersResponse,
        }[response.status_code](response.json())

    def createFulfillmentOrder(
        self,
        data: CreateFulfillmentOrderRequest,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders".format()
        params = {}
        response = self.request(url, method="POST", data=data.data)
        return {
            200: CreateFulfillmentOrderResponse,
            400: CreateFulfillmentOrderResponse,
            401: CreateFulfillmentOrderResponse,
            403: CreateFulfillmentOrderResponse,
            404: CreateFulfillmentOrderResponse,
            429: CreateFulfillmentOrderResponse,
            500: CreateFulfillmentOrderResponse,
            503: CreateFulfillmentOrderResponse,
        }[response.status_code](response.json())

    def getPackageTrackingDetails(
        self,
        packageNumber: int,
    ):
        url = "/fba/outbound/2020-07-01/tracking".format()
        params = {}
        if packageNumber is not None:
            params["packageNumber"] = (packageNumber,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetPackageTrackingDetailsResponse,
            400: GetPackageTrackingDetailsResponse,
            401: GetPackageTrackingDetailsResponse,
            403: GetPackageTrackingDetailsResponse,
            404: GetPackageTrackingDetailsResponse,
            429: GetPackageTrackingDetailsResponse,
            500: GetPackageTrackingDetailsResponse,
            503: GetPackageTrackingDetailsResponse,
        }[response.status_code](response.json())

    def listReturnReasonCodes(
        self,
        sellerSku: str,
        language: str,
        marketplaceId: str = None,
        sellerFulfillmentOrderId: str = None,
    ):
        url = "/fba/outbound/2020-07-01/returnReasonCodes".format()
        params = {}
        if sellerSku is not None:
            params["sellerSku"] = (sellerSku,)
        if marketplaceId is not None:
            params["marketplaceId"] = (marketplaceId,)
        if sellerFulfillmentOrderId is not None:
            params["sellerFulfillmentOrderId"] = (sellerFulfillmentOrderId,)
        if language is not None:
            params["language"] = (language,)
        response = self.request(url, method="GET", params=params)
        return {
            200: ListReturnReasonCodesResponse,
            400: ListReturnReasonCodesResponse,
            401: ListReturnReasonCodesResponse,
            403: ListReturnReasonCodesResponse,
            404: ListReturnReasonCodesResponse,
            429: ListReturnReasonCodesResponse,
            500: ListReturnReasonCodesResponse,
            503: ListReturnReasonCodesResponse,
        }[response.status_code](response.json())

    def createFulfillmentReturn(
        self,
        sellerFulfillmentOrderId: str,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders/{sellerFulfillmentOrderId}/return".format(
            sellerFulfillmentOrderId=sellerFulfillmentOrderId,
        )
        params = {}
        response = self.request(url, method="PUT", data=data.data)
        return {
            200: CreateFulfillmentReturnResponse,
            400: CreateFulfillmentReturnResponse,
            401: CreateFulfillmentReturnResponse,
            403: CreateFulfillmentReturnResponse,
            404: CreateFulfillmentReturnResponse,
            429: CreateFulfillmentReturnResponse,
            500: CreateFulfillmentReturnResponse,
            503: CreateFulfillmentReturnResponse,
        }[response.status_code](response.json())

    def getFulfillmentOrder(
        self,
        sellerFulfillmentOrderId: str,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders/{sellerFulfillmentOrderId}".format(
            sellerFulfillmentOrderId=sellerFulfillmentOrderId,
        )
        params = {}
        response = self.request(url, method="GET", params=params)
        return {
            200: GetFulfillmentOrderResponse,
            400: GetFulfillmentOrderResponse,
            401: GetFulfillmentOrderResponse,
            403: GetFulfillmentOrderResponse,
            404: GetFulfillmentOrderResponse,
            429: GetFulfillmentOrderResponse,
            500: GetFulfillmentOrderResponse,
            503: GetFulfillmentOrderResponse,
        }[response.status_code](response.json())

    def updateFulfillmentOrder(
        self,
        sellerFulfillmentOrderId: str,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders/{sellerFulfillmentOrderId}".format(
            sellerFulfillmentOrderId=sellerFulfillmentOrderId,
        )
        params = {}
        response = self.request(url, method="PUT", data=data.data)
        return {
            200: UpdateFulfillmentOrderResponse,
            400: UpdateFulfillmentOrderResponse,
            401: UpdateFulfillmentOrderResponse,
            403: UpdateFulfillmentOrderResponse,
            404: UpdateFulfillmentOrderResponse,
            429: UpdateFulfillmentOrderResponse,
            500: UpdateFulfillmentOrderResponse,
            503: UpdateFulfillmentOrderResponse,
        }[response.status_code](response.json())

    def cancelFulfillmentOrder(
        self,
        sellerFulfillmentOrderId: str,
    ):
        url = "/fba/outbound/2020-07-01/fulfillmentOrders/{sellerFulfillmentOrderId}/cancel".format(
            sellerFulfillmentOrderId=sellerFulfillmentOrderId,
        )
        params = {}
        response = self.request(url, method="PUT", data=data.data)
        return {
            200: CancelFulfillmentOrderResponse,
            400: CancelFulfillmentOrderResponse,
            401: CancelFulfillmentOrderResponse,
            403: CancelFulfillmentOrderResponse,
            404: CancelFulfillmentOrderResponse,
            429: CancelFulfillmentOrderResponse,
            500: CancelFulfillmentOrderResponse,
            503: CancelFulfillmentOrderResponse,
        }[response.status_code](response.json())

    def getFeatures(
        self,
        marketplaceId: str,
    ):
        url = "/fba/outbound/2020-07-01/features".format()
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = (marketplaceId,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetFeaturesResponse,
            400: GetFeaturesResponse,
            401: GetFeaturesResponse,
            403: GetFeaturesResponse,
            404: GetFeaturesResponse,
            429: GetFeaturesResponse,
            500: GetFeaturesResponse,
            503: GetFeaturesResponse,
        }[response.status_code](response.json())

    def getFeatureInventory(
        self,
        featureName: str,
        marketplaceId: str,
        nextToken: str = None,
    ):
        url = "/fba/outbound/2020-07-01/features/inventory/{featureName}".format(
            featureName=featureName,
        )
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = (marketplaceId,)
        if nextToken is not None:
            params["nextToken"] = (nextToken,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetFeatureInventoryResponse,
            400: GetFeatureInventoryResponse,
            401: GetFeatureInventoryResponse,
            403: GetFeatureInventoryResponse,
            404: GetFeatureInventoryResponse,
            429: GetFeatureInventoryResponse,
            500: GetFeatureInventoryResponse,
            503: GetFeatureInventoryResponse,
        }[response.status_code](response.json())

    def getFeatureSKU(
        self,
        featureName: str,
        sellerSku: str,
        marketplaceId: str,
    ):
        url = "/fba/outbound/2020-07-01/features/inventory/{featureName}/{sellerSku}".format(
            featureName=featureName,
            sellerSku=sellerSku,
        )
        params = {}
        if marketplaceId is not None:
            params["marketplaceId"] = (marketplaceId,)
        response = self.request(url, method="GET", params=params)
        return {
            200: GetFeatureSkuResponse,
            400: GetFeatureSkuResponse,
            401: GetFeatureSkuResponse,
            403: GetFeatureSkuResponse,
            404: GetFeatureSkuResponse,
            429: GetFeatureSkuResponse,
            500: GetFeatureSkuResponse,
            503: GetFeatureSkuResponse,
        }[response.status_code](response.json())



from .base import BaseClient as __BaseClient
from typing import List as _List


class Error:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "code" in data:
            self.code: str = str(data["code"])
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = str(data["message"])
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = str(data["details"])
        else:
            self.details: str = None


class ErrorList:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "errors" in data:
            self.errors: _List[Error] = [Error(datum) for datum in data["errors"]]
        else:
            self.errors: _List[Error] = []


class Item:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "asin" in data:
            self.asin: ItemAsin = ItemAsin(data["asin"])
        else:
            self.asin: ItemAsin = None
        if "attributes" in data:
            self.attributes: ItemAttributes = ItemAttributes(data["attributes"])
        else:
            self.attributes: ItemAttributes = None
        if "identifiers" in data:
            self.identifiers: ItemIdentifiers = ItemIdentifiers(data["identifiers"])
        else:
            self.identifiers: ItemIdentifiers = None
        if "images" in data:
            self.images: ItemImages = ItemImages(data["images"])
        else:
            self.images: ItemImages = None
        if "productTypes" in data:
            self.productTypes: ItemProductTypes = ItemProductTypes(data["productTypes"])
        else:
            self.productTypes: ItemProductTypes = None
        if "salesRanks" in data:
            self.salesRanks: ItemSalesRanks = ItemSalesRanks(data["salesRanks"])
        else:
            self.salesRanks: ItemSalesRanks = None
        if "summaries" in data:
            self.summaries: ItemSummaries = ItemSummaries(data["summaries"])
        else:
            self.summaries: ItemSummaries = None
        if "variations" in data:
            self.variations: ItemVariations = ItemVariations(data["variations"])
        else:
            self.variations: ItemVariations = None
        if "vendorDetails" in data:
            self.vendorDetails: ItemVendorDetails = ItemVendorDetails(data["vendorDetails"])
        else:
            self.vendorDetails: ItemVendorDetails = None


class ItemAttributes:
    def __init__(self, data):
        super().__init__()
        self.data = data


class ItemIdentifiersByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "identifiers" in data:
            self.identifiers: _List[ItemIdentifier] = [ItemIdentifier(datum) for datum in data["identifiers"]]
        else:
            self.identifiers: _List[ItemIdentifier] = []


class ItemIdentifier:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "identifierType" in data:
            self.identifierType: str = str(data["identifierType"])
        else:
            self.identifierType: str = None
        if "identifier" in data:
            self.identifier: str = str(data["identifier"])
        else:
            self.identifier: str = None


class ItemImagesByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "images" in data:
            self.images: _List[ItemImage] = [ItemImage(datum) for datum in data["images"]]
        else:
            self.images: _List[ItemImage] = []


class ItemImage:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "variant" in data:
            self.variant: str = str(data["variant"])
        else:
            self.variant: str = None
        if "link" in data:
            self.link: str = str(data["link"])
        else:
            self.link: str = None
        if "height" in data:
            self.height: int = int(data["height"])
        else:
            self.height: int = None
        if "width" in data:
            self.width: int = int(data["width"])
        else:
            self.width: int = None


class ItemProductTypeByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "productType" in data:
            self.productType: str = str(data["productType"])
        else:
            self.productType: str = None


class ItemSalesRanksByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "ranks" in data:
            self.ranks: _List[ItemSalesRank] = [ItemSalesRank(datum) for datum in data["ranks"]]
        else:
            self.ranks: _List[ItemSalesRank] = []


class ItemSalesRank:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "title" in data:
            self.title: str = str(data["title"])
        else:
            self.title: str = None
        if "link" in data:
            self.link: str = str(data["link"])
        else:
            self.link: str = None
        if "rank" in data:
            self.rank: int = int(data["rank"])
        else:
            self.rank: int = None


class ItemSummaryByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "brandName" in data:
            self.brandName: str = str(data["brandName"])
        else:
            self.brandName: str = None
        if "browseNode" in data:
            self.browseNode: str = str(data["browseNode"])
        else:
            self.browseNode: str = None
        if "colorName" in data:
            self.colorName: str = str(data["colorName"])
        else:
            self.colorName: str = None
        if "itemName" in data:
            self.itemName: str = str(data["itemName"])
        else:
            self.itemName: str = None
        if "manufacturer" in data:
            self.manufacturer: str = str(data["manufacturer"])
        else:
            self.manufacturer: str = None
        if "modelNumber" in data:
            self.modelNumber: str = str(data["modelNumber"])
        else:
            self.modelNumber: str = None
        if "sizeName" in data:
            self.sizeName: str = str(data["sizeName"])
        else:
            self.sizeName: str = None
        if "styleName" in data:
            self.styleName: str = str(data["styleName"])
        else:
            self.styleName: str = None


class ItemVariationsByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "asins" in data:
            self.asins: _List[str] = [str(datum) for datum in data["asins"]]
        else:
            self.asins: _List[str] = []
        if "variationType" in data:
            self.variationType: str = str(data["variationType"])
        else:
            self.variationType: str = None


class ItemVendorDetailsByMarketplace:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "marketplaceId" in data:
            self.marketplaceId: str = str(data["marketplaceId"])
        else:
            self.marketplaceId: str = None
        if "brandCode" in data:
            self.brandCode: str = str(data["brandCode"])
        else:
            self.brandCode: str = None
        if "categoryCode" in data:
            self.categoryCode: str = str(data["categoryCode"])
        else:
            self.categoryCode: str = None
        if "manufacturerCode" in data:
            self.manufacturerCode: str = str(data["manufacturerCode"])
        else:
            self.manufacturerCode: str = None
        if "manufacturerCodeParent" in data:
            self.manufacturerCodeParent: str = str(data["manufacturerCodeParent"])
        else:
            self.manufacturerCodeParent: str = None
        if "productGroup" in data:
            self.productGroup: str = str(data["productGroup"])
        else:
            self.productGroup: str = None
        if "replenishmentCategory" in data:
            self.replenishmentCategory: str = str(data["replenishmentCategory"])
        else:
            self.replenishmentCategory: str = None
        if "subcategoryCode" in data:
            self.subcategoryCode: str = str(data["subcategoryCode"])
        else:
            self.subcategoryCode: str = None


class ItemIdentifiers(list, _List["ItemIdentifiersByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemIdentifiersByMarketplace(datum) for datum in data])
        self.data = data


class ItemImages(list, _List["ItemImagesByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemImagesByMarketplace(datum) for datum in data])
        self.data = data


class ItemProductTypes(list, _List["ItemProductTypeByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemProductTypeByMarketplace(datum) for datum in data])
        self.data = data


class ItemSalesRanks(list, _List["ItemSalesRanksByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemSalesRanksByMarketplace(datum) for datum in data])
        self.data = data


class ItemSummaries(list, _List["ItemSummaryByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemSummaryByMarketplace(datum) for datum in data])
        self.data = data


class ItemVariations(list, _List["ItemVariationsByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemVariationsByMarketplace(datum) for datum in data])
        self.data = data


class ItemVendorDetails(list, _List["ItemVendorDetailsByMarketplace"]):
    def __init__(self, data):
        super().__init__([ItemVendorDetailsByMarketplace(datum) for datum in data])
        self.data = data


class ItemAsin(str):
    pass


class CatalogItems20201201Client(__BaseClient):
    def getCatalogItem(
        self,
        asin: str,
        marketplaceIds: _List[str],
        includedData: _List[str] = None,
    ):
        url = "/catalog/2020-12-01/items/{asin}".format(
            asin=asin,
        )
        params = {}
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        if includedData is not None:
            params["includedData"] = ",".join(map(str, includedData))
        response = self.request(url, method="GET", params=params)
        return {
            200: Item,
            400: ErrorList,
            403: ErrorList,
            404: ErrorList,
            413: ErrorList,
            415: ErrorList,
            429: ErrorList,
            500: ErrorList,
            503: ErrorList,
        }[response.status_code](response.json())



from .base import BaseClient as __BaseClient
from typing import List as _List


class Granularity:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "granularityType" in data:
            self.granularityType: str = str(data["granularityType"])
        else:
            self.granularityType: str = None
        if "granularityId" in data:
            self.granularityId: str = str(data["granularityId"])
        else:
            self.granularityId: str = None


class ReservedQuantity:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "totalReservedQuantity" in data:
            self.totalReservedQuantity: int = int(data["totalReservedQuantity"])
        else:
            self.totalReservedQuantity: int = None
        if "pendingCustomerOrderQuantity" in data:
            self.pendingCustomerOrderQuantity: int = int(data["pendingCustomerOrderQuantity"])
        else:
            self.pendingCustomerOrderQuantity: int = None
        if "pendingTransshipmentQuantity" in data:
            self.pendingTransshipmentQuantity: int = int(data["pendingTransshipmentQuantity"])
        else:
            self.pendingTransshipmentQuantity: int = None
        if "fcProcessingQuantity" in data:
            self.fcProcessingQuantity: int = int(data["fcProcessingQuantity"])
        else:
            self.fcProcessingQuantity: int = None


class ResearchingQuantityEntry:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "name" in data:
            self.name: str = str(data["name"])
        else:
            self.name: str = None
        if "quantity" in data:
            self.quantity: int = int(data["quantity"])
        else:
            self.quantity: int = None


class ResearchingQuantity:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "totalResearchingQuantity" in data:
            self.totalResearchingQuantity: int = int(data["totalResearchingQuantity"])
        else:
            self.totalResearchingQuantity: int = None
        if "researchingQuantityBreakdown" in data:
            self.researchingQuantityBreakdown: _List[ResearchingQuantityEntry] = [
                ResearchingQuantityEntry(datum) for datum in data["researchingQuantityBreakdown"]
            ]
        else:
            self.researchingQuantityBreakdown: _List[ResearchingQuantityEntry] = []


class UnfulfillableQuantity:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "totalUnfulfillableQuantity" in data:
            self.totalUnfulfillableQuantity: int = int(data["totalUnfulfillableQuantity"])
        else:
            self.totalUnfulfillableQuantity: int = None
        if "customerDamagedQuantity" in data:
            self.customerDamagedQuantity: int = int(data["customerDamagedQuantity"])
        else:
            self.customerDamagedQuantity: int = None
        if "warehouseDamagedQuantity" in data:
            self.warehouseDamagedQuantity: int = int(data["warehouseDamagedQuantity"])
        else:
            self.warehouseDamagedQuantity: int = None
        if "distributorDamagedQuantity" in data:
            self.distributorDamagedQuantity: int = int(data["distributorDamagedQuantity"])
        else:
            self.distributorDamagedQuantity: int = None
        if "carrierDamagedQuantity" in data:
            self.carrierDamagedQuantity: int = int(data["carrierDamagedQuantity"])
        else:
            self.carrierDamagedQuantity: int = None
        if "defectiveQuantity" in data:
            self.defectiveQuantity: int = int(data["defectiveQuantity"])
        else:
            self.defectiveQuantity: int = None
        if "expiredQuantity" in data:
            self.expiredQuantity: int = int(data["expiredQuantity"])
        else:
            self.expiredQuantity: int = None


class InventoryDetails:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "fulfillableQuantity" in data:
            self.fulfillableQuantity: int = int(data["fulfillableQuantity"])
        else:
            self.fulfillableQuantity: int = None
        if "inboundWorkingQuantity" in data:
            self.inboundWorkingQuantity: int = int(data["inboundWorkingQuantity"])
        else:
            self.inboundWorkingQuantity: int = None
        if "inboundShippedQuantity" in data:
            self.inboundShippedQuantity: int = int(data["inboundShippedQuantity"])
        else:
            self.inboundShippedQuantity: int = None
        if "inboundReceivingQuantity" in data:
            self.inboundReceivingQuantity: int = int(data["inboundReceivingQuantity"])
        else:
            self.inboundReceivingQuantity: int = None
        if "reservedQuantity" in data:
            self.reservedQuantity: ReservedQuantity = ReservedQuantity(data["reservedQuantity"])
        else:
            self.reservedQuantity: ReservedQuantity = None
        if "researchingQuantity" in data:
            self.researchingQuantity: ResearchingQuantity = ResearchingQuantity(data["researchingQuantity"])
        else:
            self.researchingQuantity: ResearchingQuantity = None
        if "unfulfillableQuantity" in data:
            self.unfulfillableQuantity: UnfulfillableQuantity = UnfulfillableQuantity(data["unfulfillableQuantity"])
        else:
            self.unfulfillableQuantity: UnfulfillableQuantity = None


class InventorySummary:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "asin" in data:
            self.asin: str = str(data["asin"])
        else:
            self.asin: str = None
        if "fnSku" in data:
            self.fnSku: str = str(data["fnSku"])
        else:
            self.fnSku: str = None
        if "sellerSku" in data:
            self.sellerSku: str = str(data["sellerSku"])
        else:
            self.sellerSku: str = None
        if "condition" in data:
            self.condition: str = str(data["condition"])
        else:
            self.condition: str = None
        if "inventoryDetails" in data:
            self.inventoryDetails: InventoryDetails = InventoryDetails(data["inventoryDetails"])
        else:
            self.inventoryDetails: InventoryDetails = None
        if "lastUpdatedTime" in data:
            self.lastUpdatedTime: str = str(data["lastUpdatedTime"])
        else:
            self.lastUpdatedTime: str = None
        if "productName" in data:
            self.productName: str = str(data["productName"])
        else:
            self.productName: str = None
        if "totalQuantity" in data:
            self.totalQuantity: int = int(data["totalQuantity"])
        else:
            self.totalQuantity: int = None


class Pagination:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "nextToken" in data:
            self.nextToken: str = str(data["nextToken"])
        else:
            self.nextToken: str = None


class GetInventorySummariesResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "granularity" in data:
            self.granularity: Granularity = Granularity(data["granularity"])
        else:
            self.granularity: Granularity = None
        if "inventorySummaries" in data:
            self.inventorySummaries: InventorySummaries = InventorySummaries(data["inventorySummaries"])
        else:
            self.inventorySummaries: InventorySummaries = None


class GetInventorySummariesResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetInventorySummariesResult = GetInventorySummariesResult(data["payload"])
        else:
            self.payload: GetInventorySummariesResult = None
        if "pagination" in data:
            self.pagination: Pagination = Pagination(data["pagination"])
        else:
            self.pagination: Pagination = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class Error:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "code" in data:
            self.code: str = str(data["code"])
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = str(data["message"])
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = str(data["details"])
        else:
            self.details: str = None


class InventorySummaries(list, _List["InventorySummary"]):
    def __init__(self, data):
        super().__init__([InventorySummary(datum) for datum in data])
        self.data = data


class ErrorList(list, _List["Error"]):
    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class FbaInventoryV1Client(__BaseClient):
    def getInventorySummaries(
        self,
        granularityType: str,
        granularityId: str,
        marketplaceIds: _List[str],
        details: bool = None,
        startDateTime: str = None,
        sellerSkus: _List[str] = None,
        nextToken: str = None,
    ):
        url = "/fba/inventory/v1/summaries".format()
        params = {}
        if details is not None:
            params["details"] = (details,)
        if granularityType is not None:
            params["granularityType"] = (granularityType,)
        if granularityId is not None:
            params["granularityId"] = (granularityId,)
        if startDateTime is not None:
            params["startDateTime"] = (startDateTime,)
        if sellerSkus is not None:
            params["sellerSkus"] = ",".join(map(str, sellerSkus))
        if nextToken is not None:
            params["nextToken"] = (nextToken,)
        if marketplaceIds is not None:
            params["marketplaceIds"] = ",".join(map(str, marketplaceIds))
        response = self.request(url, method="GET", params=params)
        return {
            200: GetInventorySummariesResponse,
            400: GetInventorySummariesResponse,
            403: GetInventorySummariesResponse,
            404: GetInventorySummariesResponse,
            429: GetInventorySummariesResponse,
            500: GetInventorySummariesResponse,
            503: GetInventorySummariesResponse,
        }[response.status_code](response.json())



from .base import BaseClient as __BaseClient
from typing import List as _List


class GetMyFeesEstimateRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "FeesEstimateRequest" in data:
            self.FeesEstimateRequest: FeesEstimateRequest = FeesEstimateRequest(data["FeesEstimateRequest"])
        else:
            self.FeesEstimateRequest: FeesEstimateRequest = None


class FeesEstimateRequest:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceId" in data:
            self.MarketplaceId: str = str(data["MarketplaceId"])
        else:
            self.MarketplaceId: str = None
        if "IsAmazonFulfilled" in data:
            self.IsAmazonFulfilled: bool = bool(data["IsAmazonFulfilled"])
        else:
            self.IsAmazonFulfilled: bool = None
        if "PriceToEstimateFees" in data:
            self.PriceToEstimateFees: PriceToEstimateFees = PriceToEstimateFees(data["PriceToEstimateFees"])
        else:
            self.PriceToEstimateFees: PriceToEstimateFees = None
        if "Identifier" in data:
            self.Identifier: str = str(data["Identifier"])
        else:
            self.Identifier: str = None
        if "OptionalFulfillmentProgram" in data:
            self.OptionalFulfillmentProgram: OptionalFulfillmentProgram = OptionalFulfillmentProgram(
                data["OptionalFulfillmentProgram"]
            )
        else:
            self.OptionalFulfillmentProgram: OptionalFulfillmentProgram = None


class GetMyFeesEstimateResponse:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "payload" in data:
            self.payload: GetMyFeesEstimateResult = GetMyFeesEstimateResult(data["payload"])
        else:
            self.payload: GetMyFeesEstimateResult = None
        if "errors" in data:
            self.errors: ErrorList = ErrorList(data["errors"])
        else:
            self.errors: ErrorList = None


class GetMyFeesEstimateResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "FeesEstimateResult" in data:
            self.FeesEstimateResult: FeesEstimateResult = FeesEstimateResult(data["FeesEstimateResult"])
        else:
            self.FeesEstimateResult: FeesEstimateResult = None


class Points:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "PointsNumber" in data:
            self.PointsNumber: int = int(data["PointsNumber"])
        else:
            self.PointsNumber: int = None
        if "PointsMonetaryValue" in data:
            self.PointsMonetaryValue: MoneyType = MoneyType(data["PointsMonetaryValue"])
        else:
            self.PointsMonetaryValue: MoneyType = None


class Error:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "code" in data:
            self.code: str = str(data["code"])
        else:
            self.code: str = None
        if "message" in data:
            self.message: str = str(data["message"])
        else:
            self.message: str = None
        if "details" in data:
            self.details: str = str(data["details"])
        else:
            self.details: str = None


class FeesEstimateResult:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "Status" in data:
            self.Status: str = str(data["Status"])
        else:
            self.Status: str = None
        if "FeesEstimateIdentifier" in data:
            self.FeesEstimateIdentifier: FeesEstimateIdentifier = FeesEstimateIdentifier(data["FeesEstimateIdentifier"])
        else:
            self.FeesEstimateIdentifier: FeesEstimateIdentifier = None
        if "FeesEstimate" in data:
            self.FeesEstimate: FeesEstimate = FeesEstimate(data["FeesEstimate"])
        else:
            self.FeesEstimate: FeesEstimate = None
        if "Error" in data:
            self.Error: FeesEstimateError = FeesEstimateError(data["Error"])
        else:
            self.Error: FeesEstimateError = None


class FeesEstimateIdentifier:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "MarketplaceId" in data:
            self.MarketplaceId: str = str(data["MarketplaceId"])
        else:
            self.MarketplaceId: str = None
        if "SellerId" in data:
            self.SellerId: str = str(data["SellerId"])
        else:
            self.SellerId: str = None
        if "IdType" in data:
            self.IdType: str = str(data["IdType"])
        else:
            self.IdType: str = None
        if "IdValue" in data:
            self.IdValue: str = str(data["IdValue"])
        else:
            self.IdValue: str = None
        if "IsAmazonFulfilled" in data:
            self.IsAmazonFulfilled: bool = bool(data["IsAmazonFulfilled"])
        else:
            self.IsAmazonFulfilled: bool = None
        if "PriceToEstimateFees" in data:
            self.PriceToEstimateFees: PriceToEstimateFees = PriceToEstimateFees(data["PriceToEstimateFees"])
        else:
            self.PriceToEstimateFees: PriceToEstimateFees = None
        if "SellerInputIdentifier" in data:
            self.SellerInputIdentifier: str = str(data["SellerInputIdentifier"])
        else:
            self.SellerInputIdentifier: str = None
        if "OptionalFulfillmentProgram" in data:
            self.OptionalFulfillmentProgram: OptionalFulfillmentProgram = OptionalFulfillmentProgram(
                data["OptionalFulfillmentProgram"]
            )
        else:
            self.OptionalFulfillmentProgram: OptionalFulfillmentProgram = None


class PriceToEstimateFees:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "ListingPrice" in data:
            self.ListingPrice: MoneyType = MoneyType(data["ListingPrice"])
        else:
            self.ListingPrice: MoneyType = None
        if "Shipping" in data:
            self.Shipping: MoneyType = MoneyType(data["Shipping"])
        else:
            self.Shipping: MoneyType = None
        if "Points" in data:
            self.Points: Points = Points(data["Points"])
        else:
            self.Points: Points = None


class FeesEstimate:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "TimeOfFeesEstimation" in data:
            self.TimeOfFeesEstimation: str = str(data["TimeOfFeesEstimation"])
        else:
            self.TimeOfFeesEstimation: str = None
        if "TotalFeesEstimate" in data:
            self.TotalFeesEstimate: MoneyType = MoneyType(data["TotalFeesEstimate"])
        else:
            self.TotalFeesEstimate: MoneyType = None
        if "FeeDetailList" in data:
            self.FeeDetailList: FeeDetailList = FeeDetailList(data["FeeDetailList"])
        else:
            self.FeeDetailList: FeeDetailList = None


class FeesEstimateError:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "Type" in data:
            self.Type: str = str(data["Type"])
        else:
            self.Type: str = None
        if "Code" in data:
            self.Code: str = str(data["Code"])
        else:
            self.Code: str = None
        if "Message" in data:
            self.Message: str = str(data["Message"])
        else:
            self.Message: str = None
        if "Detail" in data:
            self.Detail: FeesEstimateErrorDetail = FeesEstimateErrorDetail(data["Detail"])
        else:
            self.Detail: FeesEstimateErrorDetail = None


class FeeDetail:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "FeeType" in data:
            self.FeeType: str = str(data["FeeType"])
        else:
            self.FeeType: str = None
        if "FeeAmount" in data:
            self.FeeAmount: MoneyType = MoneyType(data["FeeAmount"])
        else:
            self.FeeAmount: MoneyType = None
        if "FeePromotion" in data:
            self.FeePromotion: MoneyType = MoneyType(data["FeePromotion"])
        else:
            self.FeePromotion: MoneyType = None
        if "TaxAmount" in data:
            self.TaxAmount: MoneyType = MoneyType(data["TaxAmount"])
        else:
            self.TaxAmount: MoneyType = None
        if "FinalFee" in data:
            self.FinalFee: MoneyType = MoneyType(data["FinalFee"])
        else:
            self.FinalFee: MoneyType = None
        if "IncludedFeeDetailList" in data:
            self.IncludedFeeDetailList: IncludedFeeDetailList = IncludedFeeDetailList(data["IncludedFeeDetailList"])
        else:
            self.IncludedFeeDetailList: IncludedFeeDetailList = None


class IncludedFeeDetail:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "FeeType" in data:
            self.FeeType: str = str(data["FeeType"])
        else:
            self.FeeType: str = None
        if "FeeAmount" in data:
            self.FeeAmount: MoneyType = MoneyType(data["FeeAmount"])
        else:
            self.FeeAmount: MoneyType = None
        if "FeePromotion" in data:
            self.FeePromotion: MoneyType = MoneyType(data["FeePromotion"])
        else:
            self.FeePromotion: MoneyType = None
        if "TaxAmount" in data:
            self.TaxAmount: MoneyType = MoneyType(data["TaxAmount"])
        else:
            self.TaxAmount: MoneyType = None
        if "FinalFee" in data:
            self.FinalFee: MoneyType = MoneyType(data["FinalFee"])
        else:
            self.FinalFee: MoneyType = None


class MoneyType:
    def __init__(self, data):
        super().__init__()
        self.data = data
        if "CurrencyCode" in data:
            self.CurrencyCode: str = str(data["CurrencyCode"])
        else:
            self.CurrencyCode: str = None
        if "Amount" in data:
            self.Amount: float = float(data["Amount"])
        else:
            self.Amount: float = None


class ErrorList(list, _List["Error"]):
    def __init__(self, data):
        super().__init__([Error(datum) for datum in data])
        self.data = data


class FeeDetailList(list, _List["FeeDetail"]):
    def __init__(self, data):
        super().__init__([FeeDetail(datum) for datum in data])
        self.data = data


class FeesEstimateErrorDetail(list, _List["dict"]):
    def __init__(self, data):
        super().__init__([dict(datum) for datum in data])
        self.data = data


class IncludedFeeDetailList(list, _List["IncludedFeeDetail"]):
    def __init__(self, data):
        super().__init__([IncludedFeeDetail(datum) for datum in data])
        self.data = data


class OptionalFulfillmentProgram(str):
    pass


class ProductFeesV0Client(__BaseClient):
    def getMyFeesEstimateForSKU(
        self,
        data: GetMyFeesEstimateRequest,
        SellerSKU: str,
    ):
        url = "/products/fees/v0/listings/{SellerSKU}/feesEstimate".format(
            SellerSKU=SellerSKU,
        )
        params = {}
        response = self.request(url, method="POST", data=data.data)
        return {
            200: GetMyFeesEstimateResponse,
            400: GetMyFeesEstimateResponse,
            401: GetMyFeesEstimateResponse,
            403: GetMyFeesEstimateResponse,
            404: GetMyFeesEstimateResponse,
            429: GetMyFeesEstimateResponse,
            500: GetMyFeesEstimateResponse,
            503: GetMyFeesEstimateResponse,
        }[response.status_code](response.json())

    def getMyFeesEstimateForASIN(
        self,
        data: GetMyFeesEstimateRequest,
        Asin: str,
    ):
        url = "/products/fees/v0/items/{Asin}/feesEstimate".format(
            Asin=Asin,
        )
        params = {}
        response = self.request(url, method="POST", data=data.data)
        return {
            200: GetMyFeesEstimateResponse,
            400: GetMyFeesEstimateResponse,
            401: GetMyFeesEstimateResponse,
            403: GetMyFeesEstimateResponse,
            404: GetMyFeesEstimateResponse,
            429: GetMyFeesEstimateResponse,
            500: GetMyFeesEstimateResponse,
            503: GetMyFeesEstimateResponse,
        }[response.status_code](response.json())
